
import "WalmPlugins.BandOfFarmers.ConfigDialog";
import "WalmPlugins.BandOfFarmers.TabButton";
import "WalmPlugins.BandOfFarmers.PlayerGoldNode";
import "WalmPlugins.BandOfFarmers.CraftGlobalNode";
import "WalmPlugins.BandOfFarmers.FarmNode";
import "WalmPlugins.BandOfFarmers.PlayerTraitNode";
import "WalmPlugins.BandOfFarmers.CraftNode";
import "WalmPlugins.BandOfFarmers.PlayerMessageNode";
import "WalmPlugins.BandOfFarmers.TokenSearchNode";
import "WalmPlugins.BandOfFarmers.OptionsSlider";

BandOfFarmers = class( Turbine.UI.Lotro.Window );

function BandOfFarmers:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self );
	
	self:SetSize( settings.mwindow_width or 390, settings.mwindow_height or 300 );
	self:SetPosition( settings.main_x, settings.main_y);
	self:SetVisible(false);
	
	self:SetText( "BOF - Band Of Farmers" );	
	
	self.playersData = DataLoad(Turbine.DataScope.Server, "BandOfFarmers_characters", ptr);
	if(self.playersData == nil) then
		self.playersData = {};
	end	

	-- Build GUI and logicals
	self:CreateComponents(self.playersData);
	
	
	self.resizer = Turbine.UI.Label();
	self.resizer:SetParent( self );
	self.resizer:SetSize(45, 45);
	self.resizer:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	self.resizer:SetBackground("WalmPlugins/BandOfFarmers/pictures/panel_resize_icon.tga");
	
	self.resizer:SetZOrder(self:GetZOrder()+10);	

	self.resizer.MouseDown = function( sender, args )
		if(args.Button == Turbine.UI.MouseButton.Left) then
			sender.dragStartX = args.X;
			sender.dragStartY = args.Y;
			sender.dragging = true;
			sender.dragged = false;
		end
	end
	self.resizer.MouseUp = function( sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then			
			if (sender.dragging) then
				sender.dragging = false;
			end
			settings.mwindow_height = self:GetHeight();
			settings.mwindow_width = self:GetWidth();
			Savesettings();
		end
	end
	self.resizer.MouseMove = function(sender,args)
		if ( sender.dragging ) then
			local height = self:GetHeight() + args.Y - sender.dragStartY;
			local width = self:GetWidth() + args.X - sender.dragStartX;
			if(height>300) then 
				self:SetHeight(height);
				self:UpdateResponsive();
			end
			if(width>390) then 
				self:SetWidth(width);
				self:UpdateResponsive();
			end
			self:UpdateResponsive();
		end
	end

	self.configBtn = Turbine.UI.Label();
	self.configBtn:SetParent( self );
	self.configBtn:SetBackground( "WalmPlugins/BandOfFarmers/pictures/config.jpg" );
	self.configBtn:SetSize(16, 16);
	self.configBtn.tooltip = GenericTooltip(self.configBtn, "Configuration", _("CONFIG_BTN"));	
	self.configBtn.MouseClick = function(sender, args)
		local optionDialog = BuildConfigDialog();
		optionDialog:SetVisible(true);		
	end	
	
	self.PositionChanged = function(sender, args)
		settings.main_x, settings.main_y = self:GetPosition();
		Savesettings();
	end	
	
	self.Closed = function(sender, args)
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>".._("CLOSENOTIF").."</rgb>");
		self:Stop();
	end
	
	function ptr(sender, args)
		--
	end

	self:UpdateLocalPlayerData(settings.updateAtLaunch~=false);
	self:ReloadTreeView(settings.displayedTab);
	Turbine.Chat.commWB_BOF_OnUpdate = function(data)
		self:UpdateLocalPlayerData(true);
	end
	--self:UpdateResponsive();
	--self:ReloadTreeView(settings.displayedTab);
end

function BandOfFarmers:CreateComponents(players)
	self:CreateTreeViewComponents();
	self:CreateHeaderComponents();
	
	local left = 10;
	local space = 2;
	self.goldTab = TabButton(_("GOLD"), true);
	self.goldTab:SetParent(self);
	self.goldTab:SetPosition( left, 38 );
	self.craftTab = TabButton(_("CRAFT"), true);
	self.craftTab:SetParent(self);
	self.craftTab:SetPosition( left+1*(60+space), 38 );
	self.tokensTab = TabButton(_("TOKENS"), true);
	self.tokensTab:SetParent(self);
	self.tokensTab:SetPosition( left+2*(60+space), 38 );	
	self.memoTab = TabButton(_("MEMO"), true);
	self.memoTab:SetParent(self);
	self.memoTab:SetPosition( left+5*(60+space), 38 );
	self.traitTab = TabButton("Traits", true);
	self.traitTab:SetParent(self);
	self.traitTab:SetPosition( left+4*(60+space), 38 );
	self.farmTab = TabButton("Farm", true);
	self.farmTab:SetParent(self);
	self.farmTab:SetPosition( left+3*(60+space), 38 );
	
	
	self.goldTab.MouseClick = function()
		self:ReloadTreeView(1);
	end	
	self.craftTab.MouseClick = function()
		self:ReloadTreeView(2);
	end	
	self.memoTab.MouseClick = function()
		self:ReloadTreeView(3);
	end
	-- self.stuffTab.MouseClick = function()
		-- self:ReloadTreeView(4);
	-- end	
	self.tokensTab.MouseClick = function()
		self:ReloadTreeView(5);
	end	
	self.traitTab.MouseClick = function()
		self:ReloadTreeView(6);
	end	
	self.farmTab.MouseClick = function()
		self:ReloadTreeView(7);
	end	
	if(settings.displayedTab==nil) then settings.displayedTab=1; end
	self:UpdateTabStatus();
end

function BandOfFarmers:UpdateTabStatus()
	self.goldTab:SetSelected(settings.displayedTab==1);
	self.craftTab:SetSelected(settings.displayedTab==2);
	self.memoTab:SetSelected(settings.displayedTab==3);
	-- self.stuffTab:SetSelected(settings.displayedTab==4);
	self.tokensTab:SetSelected(settings.displayedTab==5);
	self.traitTab:SetSelected(settings.displayedTab==6);
	self.farmTab:SetSelected(settings.displayedTab==7);
	self.optionsSlider:SetMode(settings.displayedTab);
end

function BandOfFarmers:UpdateResponsive()
	self.resizer:SetPosition(self:GetWidth()-45, self:GetHeight()-45);
	self.reloadButton:SetPosition( (self:GetWidth()-self.reloadButton:GetWidth())/2, self:GetHeight()-32 );
	self.configBtn:SetPosition(self.reloadButton:GetWidth()+self.reloadButton:GetLeft()+10, self.reloadButton:GetTop()+2);	
	self.upperSep:SetSize( self:GetWidth()-8 , 2 );
	self.lowerSep:SetSize( self:GetWidth()-8 , 2 );
	self.lowerSep:SetPosition( 4, self:GetHeight()-40 );

	self.treeView:SetSize( self:GetWidth()-24, self:GetHeight()-100 );
    self.treeViewScrollBar:SetPosition(self:GetWidth()-15, self.treeView:GetTop() );
    self.treeViewScrollBar:SetSize( 10, self.treeView:GetHeight() );
	self.optionsSlider:SetPosition(4, self.lowerSep:GetTop()-20 ) ;

	for i=1 , self.treeView:GetNodes():GetCount() , 1 do
		local node = self.treeView:GetNodes():Get(i);
		node:UpdateResponsive(self.treeView:GetWidth(), node:GetHeight());
	end	
end


function BandOfFarmers:ReloadTreeView(id, players)
	if(players==nil) then players = self.playersData; end;
	if(self.localPlayerName==nil) then self.localPlayerName=Turbine.Gameplay.LocalPlayer:GetInstance():GetName(); end
	-- if(id == self.showingId) then
		-- self:UpdateTreeViewData(nil);
	-- else
	self.treeView:GetNodes():Clear();
	if(id==nil) then id=settings.displayedTab; end
	if(id==1 or id==nil) then
		local totalPlayer = {}; 
		totalPlayer.name = _("TREEGOLD");
		totalPlayer.globalgold = true;
		totalPlayer.data = {};
		totalPlayer.data.gold = 0;
		local totalgold = 0;
		for i=1 , table.getn(self.playersData) , 1 do
			totalPlayer.data.gold = totalPlayer.data.gold + (self.playersData[i].data.gold or 0);
		end			
		local totalnode = PlayerGoldNode(totalPlayer, true);
		self.treeView:GetNodes():Add(totalnode);
		
		table.sort(self.playersData, sortdatabygold);
		for i=1 , table.getn(self.playersData) , 1 do
			local playerData = self.playersData[i];
			local node = PlayerGoldNode(playerData, playerData.name==self.localPlayerName);
			self.treeView:GetNodes():Add(node);
		end
		id = 1; 
	elseif id == 2 then 
		local globalCraftNode = CraftGlobalNode(self.playersData);
		self.treeView:GetNodes():Add(globalCraftNode);
		
		table.sort(self.playersData, sortdatabyname);
		for i=1 , table.getn(self.playersData) , 1 do
			local playerData = self.playersData[i];
			if playerData.data~=nil and playerData.data.profession~=nil and table.getn(playerData.data.profession)>0 then
				local node = CraftNode(playerData, playerData.name==self.localPlayerName, self);
				self.treeView:GetNodes():Add(node);
			end
		end
	elseif id == 3 then
		table.sort(self.playersData, sortdatabyname);
		for i=1 , table.getn(self.playersData) , 1 do
			local playerData = self.playersData[i];
			local node = PlayerMessageNode(playerData, self, playerData.name==self.localPlayerName);
			self.treeView:GetNodes():Add(node);
		end
	elseif id==4 then
		-- for i=1 , table.getn(self.playersData) , 1 do
			-- if(self.playersData[i].stuff ~= nil) then
				-- local node = PlayerStuffNode(self.playersData[1]);
				-- self.treeView:GetNodes():Add(node);
			-- end
		-- end		
	elseif id==5 then
		local searchNode = TokenSearchNode(self.playersData, self.treeView);
		self.treeView:GetNodes():Add(searchNode);
	elseif id==6 then
		table.sort(self.playersData, sortdatabyname);
		for i, pData in pairs(self.playersData) do
			if(pData~=nil and ((pData.data~=nil and (pData.data.class~=nil or pData.data.forceClassCount)) or pData.name==self.localPlayerName) ) then
				local node = PlayerTraitNode(self, pData, pData.name==self.localPlayerName, i);
				self.treeView:GetNodes():Add(node);
			end
		end
	elseif id==7 then
		table.sort(self.playersData, sortdatabyname);
		for i, pData in pairs(self.playersData) do
			if(pData~=nil and ((pData.data~=nil and pData.data.class~=nil) or pData.name==self.localPlayerName)) then
				local node = FarmNode(self, pData, pData.name==self.localPlayerName, i);
				self.treeView:GetNodes():Add(node);
			end
		end
	end
	
	self.showingId = id;
	
	if(settings.displayedTab ~= id) then
		settings.displayedTab = id;
		Savesettings();
	end
	self:UpdateResponsive();
	
	-- end
	self:UpdateTabStatus();
end

function BandOfFarmers:UpdateTreeViewData(id)
	if id == nil then
		return 0;
	end
	
	for i=1 , self.treeView:GetNodes():GetCount() , 1 do
		local node = self.treeView:GetNodes():Get(i);
		if(node.GetPlayer~=nil and node:GetPlayer()~=nil and node:GetPlayer().name == id.name) then
			node:UpdateNodeData(id);
		elseif(node.GetPlayer~=nil and node:GetPlayer()~=nil and node:GetPlayer().globalgold == true) then
			local gold = 0;
			for i=1 , table.getn(self.playersData) , 1 do
				gold = gold + self.playersData[i].data.gold;
			end
			node:GetPlayer().data.gold = gold;
		end
	end
end


function BandOfFarmers:CreateTreeViewComponents()
	self.treeView = Turbine.UI.TreeView();
    self.treeView:SetParent( self );
    self.treeView:SetPosition( 10, 62 );
    self.treeView:SetIndentationWidth( 15 );
	--self.treeView:SetBackColor(Turbine.UI.Color( 0, 0, 1 ));
	self.treeViewScrollBar = Turbine.UI.Lotro.ScrollBar();
    self.treeViewScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
    self.treeViewScrollBar:SetParent( self );
	self.treeView:SetVerticalScrollBar( self.treeViewScrollBar );

	self.upperSep = Turbine.UI.Control();
	self.upperSep:SetParent( self );
	self.upperSep:SetPosition( 4, 60 );	
	self.upperSep:SetBackColor( Turbine.UI.Color(0.5, 0.5, 0.5) );
	self.upperSep:SetZOrder(99);
	self.lowerSep = Turbine.UI.Control();
	self.lowerSep:SetParent( self );
	self.lowerSep:SetBackColor( Turbine.UI.Color(0.5, 0.5, 0.5) );
	self.lowerSep:SetZOrder(99);
	local updateFarmFct = function()
		for i=1 , self.treeView:GetNodes():GetCount() , 1 do
			local node = self.treeView:GetNodes():Get(i);
			if(node.BuildTreeView~=nil)then node:BuildTreeView(); node:UpdateResponsive(); end
		end
		Savesettings();
	end
	local config = {[7]={cpm={
		{ico="tag_vertues.tga",t="checkbox",config="farm_show_vertue", defaut=true, tooltip=_("TOOLTIP_HEAD").._("R_XP")},
		{ico="tag_traceries.tga",t="checkbox",config="farm_show_traceries", defaut=true, tooltip=_("TOOLTIP_HEAD").._("R_RUNE")},
		{ico="tag_braises.tga",t="checkbox",config="farm_show_braise", defaut=true, tooltip=_("TOOLTIP_HEAD").._("R_BRAISE")},
		{ico="tag_grains.tga",t="checkbox",config="farm_show_grains", defaut=true, tooltip=_("TOOLTIP_HEAD").._("R_GRAIN")},
		{ico="tag_reput.tga",t="checkbox",config="farm_reput", defaut=true, tooltip=_("TOOLTIP_HEAD").._("R_REPUT")},
		{ico="tag_custrep.tga",t="checkbox",config="farm_show_customs", defaut=true, tooltip=_("R_CUSTOM")},
		{t="sep"},
		{ico="tag_fellow.tga",t="checkbox",config="showemptyfarm", defaut=false, tooltip=_("TOOLTIP_ALLCHAR")}}, fct=updateFarmFct}};
	self.optionsSlider = OptionsSlider(config);
	self.optionsSlider:SetParent(self);
end

function BandOfFarmers:CreateHeaderComponents()
	self.reloadButton = Turbine.UI.Lotro.Button();
	self.reloadButton:SetParent( self );
	self.reloadButton:SetText(_("UPDATEDATA"));
	self.reloadButton:SetSize( 150 , 18 );	
	self.reloadButton.Click = function()
		self:UpdateLocalPlayerData(false);
	end
end

function BandOfFarmers:UpdateLocalPlayerData(isLeasyUpdate)
	if(self.isUpdatingData==true or self.playersData.lastUpdate~=nil and self.playersData.lastUpdate > Turbine.Engine.GetGameTime()-5) then 
		--Turbine.Shell.WriteLine("<rgb=#FF5CCD>TROP DE SAUVEGARDES</rgb>"); 
		return 0;
	end;
	-- if isLeasyUpdate==nil then
		-- Turbine.Shell.WriteLine("<rgb=#FF5CCD>UpdateLocalPlayerData(nil)</rgb>");
	-- elseif isLeasyUpdate==true then
		-- Turbine.Shell.WriteLine("<rgb=#FF5CCD>UpdateLocalPlayerData(true)</rgb>");
	-- elseif isLeasyUpdate==false then
		-- Turbine.Shell.WriteLine("<rgb=#FF5CCD>UpdateLocalPlayerData(false)</rgb>");
	-- else
		-- Turbine.Shell.WriteLine("<rgb=#FF5CCD>UpdateLocalPlayerData(????)</rgb>");
	-- end
	self.isUpdatingData = true;
	local localPlayerData = self:GetLocalPlayerData(isLeasyUpdate, isLeasyUpdate);
	if localPlayerData == nil then
		self.isUpdatingData = nil;
		return 0;
	end
	local found = 0;
	for i=1 , table.getn(self.playersData) , 1 do
		local pData = self.playersData[i];
		if(pData.name == localPlayerData.name) then
			local previousMessage = self.playersData[i].message;
			if(localPlayerData.data.class==nil) then 
				localPlayerData.data.class = self.playersData[i].data.class;
			end
			if(localPlayerData.data.cd==nil) then 
				localPlayerData.data.cd = self.playersData[i].data.cd;
			end
			if(localPlayerData.data.profession==nil) then 
				localPlayerData.data.profession = self.playersData[i].data.profession;
			end
			if(localPlayerData.data.wallet==nil) then 
				localPlayerData.data.wallet = self.playersData[i].data.wallet;
			end
			if(localPlayerData.data.forceClassCount==nil) then 
				localPlayerData.data.forceClassCount = self.playersData[i].data.forceClassCount;
			end
			self.playersData[i] = localPlayerData;
			self.playersData[i].message = previousMessage;				
			found = i;
		end
	end
	if(found == 0) then
		table.insert(self.playersData, localPlayerData);
		self:ReloadTreeView(self.showingId);
	else
		self:UpdateTreeViewData(self.playersData[found]);
		self:ReloadTreeView(self.showingId);
	end
	DataSave(Turbine.DataScope.Server, "BandOfFarmers_characters", self.playersData);
	Turbine.Shell.WriteLine("<rgb=#FF5CCD>BOF - Band of Farmers : ".._("DATAOF")..localPlayerData.name.._("UPDATED").."</rgb>");
	self.isUpdatingData = nil;
end

function BandOfFarmers:GetLocalPlayerData(excludeWallet, excludeProfession)
	local player = Turbine.Gameplay.LocalPlayer:GetInstance();
	if(player == nil) then
		return nil;
	end
	self.localPlayerName = player:GetName();
	if(self.playerAttributes == nil ) then
		self.playerAttributes = player:GetAttributes();
	end
	local result = {};
	result.name = player:GetName();
	result.data = {}
	result.data.level = player:GetLevel(); -- LevelChanged(sender, args)
	result.data.playerClass = player:GetClass();
	result.data.maxMoral = player:GetMaxMorale();
	result.data.maxPower = player:GetMaxPower();
	
	local z=player:GetClass();
	if(z~=172 and z~=214 and z~=40 and z~=23 and z~=194 and z~=193 and z~=185 and z~=162 and z~=31 and z~=24 and z~=215) then
		Turbine.Shell.WriteLine("<rgb=#FF5CCD>".._("ERRCLASS")..(z or "nil").."</rgb>");
		return nil;
	end

	if(Turbine.Chat.commWB_BOF_GetWBTraitCD~=nil) then
		local traitData, cdData, customWads = Turbine.Chat.commWB_BOF_GetWBTraitCD();	
		result.data.class = traitData;
		result.data.cd = cdData;
		settings.customWads = customWads;
		if(result.data.class==nil) then  Turbine.Shell.WriteLine("<rgb=#FF5CCD>CLASS NULL</rgb>"); end
	end	
	
	result.data.gold = self.playerAttributes:GetMoney();
	
	if(excludeProfession~=true) then
		--Turbine.Shell.WriteLine("<rgb=#FF5CCD>PRO");
		result.data.profession = {};
		result.data.profession.vocation = self.playerAttributes:GetVocation();
		local profId = 1;
		for i=1, 12 , 1 do
			local job = self.playerAttributes:GetProfessionInfo(i);
			if(job ~= nil) then
				result.data.profession[profId] = {};
				result.data.profession[profId].name = job:GetName();
				result.data.profession[profId].level = job:GetProficiencyLevel();
				result.data.profession[profId].levelXP = ""..job:GetProficiencyExperience().."/"..job:GetProficiencyExperienceTarget();
				result.data.profession[profId].mastery = job:GetMasteryLevel();
				result.data.profession[profId].masteryXP = ""..job:GetMasteryExperience().."/"..job:GetMasteryExperienceTarget();
				
				--Turbine.Shell.WriteLine("<rgb=#FF5CCD>Profession trouvée : ".. result.data.profession[profId].name ..": niveau " ..result.data.profession[profId].level.." | guilde : ".. result.data.profession[profId].mastery .."  </rgb>");
				profId = profId + 1;
			end
		end
	end
	-- result.stuff = {};
	-- result.stuff[1] = player:GetEquipment():GetItem(1);
	
	if( excludeWallet~=true ) then
		--Turbine.Shell.WriteLine("<rgb=#FF5CCD>WALLET");
		local wallet = player:GetWallet();
		result.data.wallet = {};
		for i=1 , wallet:GetSize() , 1 do
			local walletItem = wallet:GetItem(i);
			if(walletItem:IsAccountItem()==false and walletItem:GetQuantity()>0) then
			--if(walletItem:GetQuantity()>0) then
				local wItem = {};
				wItem.name = walletItem:GetName();
				wItem.quantity = walletItem:GetQuantity();
				wItem.maxQuantity = walletItem:GetMaxQuantity();
				wItem.image = walletItem:GetImage();
				wItem.smallImage = walletItem:GetSmallImage();
				--wItem.isAccountItem = walletItem:IsAccountItem();
				--wItem.description = walletItem:GetDescription();
				table.insert(result.data.wallet, wItem);
			end
		end
	end
	
	---------
	-- for i=1, player:GetUntrainedSkills():GetCount(), 1 do
		-- local skill =  player:GetUntrainedSkills():GetItem(i);
		-- Turbine.Shell.WriteLine("<rgb=#FF5CCD>SKILL "..i.." : "..skill:GetSkillInfo():GetName());
	-- end
	return result;
end

function BandOfFarmers:HightlightProfession(professionName)
	for i=1 , self.treeView:GetNodes():GetCount() , 1 do
		local node = self.treeView:GetNodes():Get(i);
		node:HightlightProfession(professionName);
	end
end

function BandOfFarmers:UpdateAndSavePlayerMessage(playerName, message, height)
	--Turbine.Shell.WriteLine("<rgb=#FF5CCD>Message à sauvegarder pour "..playerName.." :\n"..message.." ["..height.."]</rgb>");
	for i=1 , table.getn(self.playersData) , 1 do
		if(self.playersData[i].name == playerName) then
			self.playersData[i].message = message;
			self.playersData[i].messageHeight = height;
			DataSave(Turbine.DataScope.Server, "BandOfFarmers_characters", self.playersData);
			--Turbine.Shell.WriteLine("<rgb=#FF5CCD>Message de "..playerName.." sauvegardé</rgb>");
		end
	end
end

function BandOfFarmers:DeleteCharacterData(playerName)
	if(playerName==nil) then return; end
	for i=1 , table.getn(self.playersData) , 1 do
		if(self.playersData[i].name == playerName) then
			table.remove(self.playersData, i);
			self:ReloadTreeView(settings.displayedTab);
			Turbine.Shell.WriteLine("<rgb=#FF5CCD>".. _("DATA_MES1")..playerName .._("DATA_MES2").."</rgb>");
			return;
		end
	end
end

function BandOfFarmers:GetName()
	return "BOF - Band Of Farmers";
end

function BandOfFarmers:IsRunning()
	return ( self ~= nil );
end

function BandOfFarmers:Start()
	if(self~=nil) then 
		self:SetVisible(true); 
	end
end

function BandOfFarmers:Stop()
	self = nil;
end

function sortdatabygold(player1,player2)
	if player1==nil or player1.data==nil or player2==nil or player2.data==nil then
		return true;
	end
	if ((player1.data.gold or 0) > (player2.data.gold or 0)) then
		return true
	end
end

function sortdatabyname(player1,player2)
	if player1== nil or player2==nil then
		return true;
	end
	if (player1.name < player2.name) then
		return true
	end
end

function SortWBByLevel(player1,player2)
	if player1==nil or player2==nil then
		return true;
	end
	if (player1.level or 0 < player2.level or 0) then
		return true
	end
end